#include "pch.h"
#include "Telemetry.h"

using namespace SenseHat::Sensors::TelemetryControl;

void Telemetry::Start()
{
	if (!IsActive)
	{
		IsActive = true;

		InitializeTelemetryTask();
	}
}

void Telemetry::Stop()
{
	if (IsActive)
	{
		telemetryCancellationTokenSource->cancel();

		IsActive = false;

		delete telemetryCancellationTokenSource;
		telemetryCancellationTokenSource = nullptr;
	}
}

void Telemetry::InitializeTelemetryTask()
{	
	telemetryCancellationTokenSource = new cancellation_token_source();

	telemetryTask = task<void>([this]
	{
		auto cancellationToken = telemetryCancellationTokenSource->get_token();
	
		while (!cancellationToken.is_canceled())
		{
			if (IsActive)
			{
				auto telemetryEventArgs = GetSensorReading();

				OnDataReady(this, telemetryEventArgs);

				Sleep(msDelayTime);
			}
		}

	}, telemetryCancellationTokenSource->get_token());	
}

TelemetryEventArgs^ Telemetry::GetSensorReading()
{
	const double baseTemp = 35.0;
	const double interval = 2.5;

	auto temp = baseTemp + interval * rand() / double(RAND_MAX);

	return ref new TelemetryEventArgs(temp);
}
