﻿using SenseHatDisplay.Helpers;
using SenseHatIO.SenseHatLedArray;
using Speech;
using System;
using System.Collections.Generic;
using System.Threading.Tasks;
using Windows.Media.SpeechRecognition;
using Windows.UI;
using Windows.UI.Xaml.Controls;

namespace SpeechControl
{
    public sealed partial class MainPage : Page
    {
        private const string welcomeMessage = "I'm ready. Choose a color";
        private const string recognitionError = "I did not get that";
        private const string confirmationMessage = "OK";

        private SpeechRecognizer speechRecognizer;
        private LedArray ledArray;

        private Dictionary<string, Color> commandsDictionary;

        public MainPage()
        {
            InitializeComponent();

            InitializeLedArray();

            InitializeCommandsDictionary();

            InitializeSpeechRecognizer();            
        }

        private async void InitializeLedArray()
        {
            const byte address = 0x46;

            var device = await I2cHelper.GetI2cDevice(address);
            ledArray = new LedArray(device);
        }

        private void InitializeCommandsDictionary()
        {
            commandsDictionary = new Dictionary<string, Color>();

            commandsDictionary.Add("Red", Colors.Red);
            commandsDictionary.Add("Green", Colors.Green);
            commandsDictionary.Add("Blue", Colors.Blue);
            commandsDictionary.Add("None", Colors.Black);
        }

        private async void InitializeSpeechRecognizer()
        {
            try
            {
                speechRecognizer = new SpeechRecognizer();

                // Configure constraints
                var listConstraint = new SpeechRecognitionListConstraint(commandsDictionary.Keys);
                speechRecognizer.Constraints.Add(listConstraint);
                await speechRecognizer.CompileConstraintsAsync();

                // Attach event handler and start continuous recognition
                speechRecognizer.ContinuousRecognitionSession.ResultGenerated +=
                    ContinuousRecognitionSession_ResultGenerated;
                
                await speechRecognizer.ContinuousRecognitionSession.StartAsync();

                SpeechHelper.Speak(welcomeMessage);
            }
            catch (UnauthorizedAccessException)
            {
                StartBlinking();
            }
        }

        private void ContinuousRecognitionSession_ResultGenerated(
            SpeechContinuousRecognitionSession sender,
            SpeechContinuousRecognitionResultGeneratedEventArgs args)
        {
            string message = recognitionError;

            if (args.Result.Confidence != SpeechRecognitionConfidence.Rejected)
            {
                message = SetColor(args);
            }

            SpeechHelper.Speak(message);
        }

        private string SetColor(SpeechContinuousRecognitionResultGeneratedEventArgs args)
        {
            var message = recognitionError;

            var recognizedText = args.Result.Text;

            if (commandsDictionary.ContainsKey(recognizedText))
            {
                var color = commandsDictionary[recognizedText];

                message = confirmationMessage;

                ledArray.Reset(color);
            }

            return message;
        }

        private void StartBlinking()
        {
            const int msDelay = 100;

            while (true)
            {
                ledArray.Reset(Colors.Black);
                Task.Delay(msDelay).Wait();

                ledArray.Reset(Colors.Red);
                Task.Delay(msDelay).Wait();
            }
        }
    }
}
