﻿using AnomalyDetection.Storage;
using SenseHat.Sensors;
using System;
using System.Collections.Generic;
using System.Threading.Tasks;

namespace AnomalyDetection.Training
{
    public class TrainingDatasetAcquisition
    {
        private TemperatureAndPressureSensor sensor = TemperatureAndPressureSensor.Instance;

        public async Task<string> Acquire(int msDelayTime, TimeSpan duration)
        {            
            // Prepare storage
            var storage = await TemperatureFileStorage.CreateAsync();

            // Initialize sensor
            await sensor.Initialize();
            
            // Begin asynchronuous sensor reading
            var temperatureDataset = new List<float>();

            await BeginSensorReading(() =>
            {
                var temp = sensor.GetTemperature();
                temperatureDataset.Add(temp);
            }, msDelayTime, duration);

            // and write resulting dataset to CSV file
            await storage.WriteData(temperatureDataset);

            return storage.FilePath;
        }

        private async Task BeginSensorReading(Action periodicAction, int msDelayTime, TimeSpan duration)
        {
            await Task.Run(() =>
            {
                var beginTime = DateTime.Now.ToUniversalTime();
                var currentTime = beginTime;

                // Perform action unless specified period of time has passed
                while (currentTime - beginTime <= duration)
                {
                    periodicAction();

                    Task.Delay(msDelayTime).Wait();

                    currentTime = DateTime.Now.ToUniversalTime();
                };
            });
        }
    }
}
