var Microsoft;
(function (Microsoft) {
    var Azure;
    (function (Azure) {
        var Devices;
        (function (Devices) {
            var Applications;
            (function (Applications) {
                var PredictiveMaintenance;
                (function (PredictiveMaintenance) {
                    var LineChart = (function () {
                        function LineChart(htmlElement) {
                            this.htmlElement = htmlElement;
                            this.createDefaultStyles = this.createDefaultStyles.bind(this);
                            this.createLineChartVisual = this.createLineChartVisual.bind(this);
                            this.resizeViewport = this.resizeViewport.bind(this);
                            this.pluginService = powerbi.visuals.visualPluginFactory.create();
                            this.singleVisualHostServices = powerbi.visuals.singleVisualHostServices;
                            this.lineChartVisual = this.createLineChartVisual();
                        }
                        LineChart.prototype.createDefaultStyles = function () {
                            var dataColors = new powerbi.visuals.DataColorPalette();
                            return {
                                titleText: {
                                    color: { value: "rgba(51,51,51,1)" }
                                },
                                subTitleText: {
                                    color: { value: "rgba(145,145,145,1)" }
                                },
                                colorPalette: {
                                    dataColors: dataColors
                                },
                                labelText: {
                                    color: {
                                        value: "rgba(51,51,51,1)"
                                    },
                                    fontSize: "12px"
                                },
                                isHighContrast: false
                            };
                        };
                        LineChart.prototype.createLineChartVisual = function () {
                            var $htmlElement = $(this.htmlElement);
                            var width = $htmlElement.width();
                            var height = $htmlElement.height();
                            var lineChartVisual = this.pluginService.getPlugin("lineChart").create();
                            lineChartVisual.init({
                                element: $htmlElement,
                                host: this.singleVisualHostServices,
                                style: this.createDefaultStyles(),
                                viewport: { height: height, width: width },
                                settings: { slicingEnabled: false },
                                interactivity: { isInteractiveLegend: false, selection: false },
                                animation: { transitionImmediate: false }
                            });
                            return lineChartVisual;
                        };
                        LineChart.prototype.updateChartData = function (lineChartDataView) {
                            this.lineChartVisual.onDataChanged({
                                dataViews: [lineChartDataView],
                                viewport: this.lineChartVisual.viewport,
                                duration: 0
                            });
                        };
                        LineChart.prototype.resizeViewport = function () {
                            this.lineChartVisual.currentViewport.width = this.htmlElement.clientWidth;
                            this.lineChartVisual.render();
                        };
                        return LineChart;
                    }());
                    PredictiveMaintenance.LineChart = LineChart;
                })(PredictiveMaintenance = Applications.PredictiveMaintenance || (Applications.PredictiveMaintenance = {}));
            })(Applications = Devices.Applications || (Devices.Applications = {}));
        })(Devices = Azure.Devices || (Azure.Devices = {}));
    })(Azure = Microsoft.Azure || (Microsoft.Azure = {}));
})(Microsoft || (Microsoft = {}));
var Microsoft;
(function (Microsoft) {
    var Azure;
    (function (Azure) {
        var Devices;
        (function (Devices) {
            var Applications;
            (function (Applications) {
                var PredictiveMaintenance;
                (function (PredictiveMaintenance) {
                    var PredictionLineChartBinding = (function () {
                        function PredictionLineChartBinding() {
                            this.init = this.init.bind(this);
                            this.createDataView = this.createDataView.bind(this);
                        }
                        PredictionLineChartBinding.prototype.createDataView = function (categories, line1values, line2values) {
                            var fieldExpr = powerbi.data.SQExprBuilder.fieldExpr({ column: { schema: "s", entity: "table2", name: "cycle" } });
                            var categoryIdentities = categories.map(function (value) {
                                var expr = powerbi.data.SQExprBuilder.equal(fieldExpr, powerbi.data.SQExprBuilder.integer(value));
                                return powerbi.data.createDataViewScopeIdentity(expr);
                            });
                            var dataViewMetadata = {
                                columns: [
                                    {
                                        displayName: "Cycle",
                                        queryName: "Cycle",
                                        isMeasure: true,
                                        format: "00",
                                        type: powerbi.ValueType.fromDescriptor({ numeric: true })
                                    },
                                    {
                                        displayName: "Engine 1",
                                        isMeasure: true,
                                        queryName: "engine1",
                                        type: powerbi.ValueType.fromDescriptor({ numeric: true }),
                                        objects: { dataPoint: { fill: { solid: { color: "#C02FB0" } } } }
                                    },
                                    {
                                        displayName: "Engine 2",
                                        isMeasure: true,
                                        queryName: "engine2",
                                        type: powerbi.ValueType.fromDescriptor({ numeric: true }),
                                        objects: { dataPoint: { fill: { solid: { color: "#00AAAD" } } } }
                                    }
                                ]
                            };
                            var columns = [
                                {
                                    source: dataViewMetadata.columns[1],
                                    values: line1values
                                },
                                {
                                    source: dataViewMetadata.columns[2],
                                    values: line2values
                                }
                            ];
                            var dataValues = powerbi.data.DataViewTransform.createValueColumns(columns);
                            var tableDataValues = categories.map(function (countryName, idx) { return [countryName, columns[0].values[idx], columns[1].values[idx]]; });
                            return {
                                metadata: dataViewMetadata,
                                categorical: {
                                    categories: [{
                                            source: dataViewMetadata.columns[0],
                                            values: categories,
                                            identity: categoryIdentities,
                                        }],
                                    values: dataValues
                                },
                                table: {
                                    rows: tableDataValues,
                                    columns: dataViewMetadata.columns
                                }
                            };
                        };
                        PredictionLineChartBinding.prototype.init = function (element, valueAccessor) {
                            var _this = this;
                            $(element).addClass("chart-indicator-progress");
                            var sourceObservable = valueAccessor();
                            var lineChart = new PredictiveMaintenance.LineChart(element);
                            var onChartDataReceived = function (chartData) {
                                if (!chartData) {
                                    return;
                                }
                                var dataView = _this.createDataView(chartData.categories, chartData.line1values, chartData.line2values);
                                lineChart.updateChartData(dataView);
                                $(element).removeClass("chart-indicator-progress");
                            };
                            sourceObservable.subscribe(onChartDataReceived);
                            $(window).resize(function () {
                                lineChart.resizeViewport();
                            });
                            onChartDataReceived(sourceObservable());
                        };
                        return PredictionLineChartBinding;
                    }());
                    PredictiveMaintenance.PredictionLineChartBinding = PredictionLineChartBinding;
                    var sensorLineChartBinding = new PredictionLineChartBinding();
                    ko.bindingHandlers["predictionLineChart"] = {
                        init: sensorLineChartBinding.init
                    };
                })(PredictiveMaintenance = Applications.PredictiveMaintenance || (Applications.PredictiveMaintenance = {}));
            })(Applications = Devices.Applications || (Devices.Applications = {}));
        })(Devices = Azure.Devices || (Azure.Devices = {}));
    })(Azure = Microsoft.Azure || (Microsoft.Azure = {}));
})(Microsoft || (Microsoft = {}));
var Microsoft;
(function (Microsoft) {
    var Azure;
    (function (Azure) {
        var Devices;
        (function (Devices) {
            var Applications;
            (function (Applications) {
                var PredictiveMaintenance;
                (function (PredictiveMaintenance) {
                    var Constants = (function () {
                        function Constants() {
                        }
                        Constants.couldNotSendCommand = "We're sorry, we could not send a command to the devices. Please try again later.";
                        Constants.couldNotLoadTelemetry = "We're sorry, we could not load telemetry data. Please try refreshing the page.";
                        Constants.couldNotLoadPredictions = "We're sorry, we could not load RUL data. Please try refreshing the page.";
                        return Constants;
                    }());
                    PredictiveMaintenance.Constants = Constants;
                })(PredictiveMaintenance = Applications.PredictiveMaintenance || (Applications.PredictiveMaintenance = {}));
            })(Applications = Devices.Applications || (Devices.Applications = {}));
        })(Devices = Azure.Devices || (Azure.Devices = {}));
    })(Azure = Microsoft.Azure || (Microsoft.Azure = {}));
})(Microsoft || (Microsoft = {}));
var Microsoft;
(function (Microsoft) {
    var Azure;
    (function (Azure) {
        var Devices;
        (function (Devices) {
            var Applications;
            (function (Applications) {
                var PredictiveMaintenance;
                (function (PredictiveMaintenance) {
                    var Dashboard = (function () {
                        function Dashboard() {
                            this.startSimulation = this.startSimulation.bind(this);
                            this.stopSimulation = this.stopSimulation.bind(this);
                            this.getTelemetryData = this.getTelemetryData.bind(this);
                            this.onTelemetryDataLoaded = this.onTelemetryDataLoaded.bind(this);
                            this.getPredictionData = this.getPredictionData.bind(this);
                            this.onPredictionDataLoaded = this.onPredictionDataLoaded.bind(this);
                            this.onTelemetryLoadError = this.onTelemetryLoadError.bind(this);
                            this.onPredictionLoadError = this.onPredictionLoadError.bind(this);
                            this.onSendCommandError = this.onSendCommandError.bind(this);
                            this.closeMessage = this.closeMessage.bind(this);
                            this.getSimulationState = this.getSimulationState.bind(this);
                            this.onSimulationStateReceived = this.onSimulationStateReceived.bind(this);
                            this.emptyString = "N/A";
                            this.warningTreshold = 160;
                            this.httpClient = new PredictiveMaintenance.JQueryHttpClient();
                            this.sensor1Data = ko.observable();
                            this.sensor2Data = ko.observable();
                            this.sensor3Data = ko.observable();
                            this.sensor4Data = ko.observable();
                            this.rulData = ko.observable();
                            this.engine1Rul = ko.observable(this.emptyString);
                            this.engine1Cycles = ko.observable(this.emptyString);
                            this.engine2Rul = ko.observable(this.emptyString);
                            this.engine2Cycles = ko.observable(this.emptyString);
                            this.engine1RulWarning = ko.observable(false);
                            this.engine2RulWarning = ko.observable(false);
                            this.simulationState = ko.observable(PredictiveMaintenance.SimulationStates.stopped);
                            this.errorMessages = ko.observableArray();
                            this.sendingCommand = ko.observable(false);
                            this.getTelemetryData();
                            this.getPredictionData();
                            setInterval(this.getSimulationState, 3000);
                        }
                        Dashboard.prototype.getSimulationState = function () {
                            var getSimulationStatePromise = this.httpClient.get("api/simulation/state");
                            getSimulationStatePromise.done(this.onSimulationStateReceived);
                        };
                        Dashboard.prototype.onSimulationStateReceived = function (state) {
                            this.simulationState(state);
                        };
                        Dashboard.prototype.getTelemetryData = function () {
                            var getTelemetryPromise = this.httpClient.get("api/telemetry");
                            getTelemetryPromise.done(this.onTelemetryDataLoaded);
                            getTelemetryPromise.fail(this.onTelemetryLoadError);
                        };
                        Dashboard.prototype.onTelemetryDataLoaded = function (enginesTelemetry) {
                            var sensor1Data = { categories: [], line1values: [], line2values: [] };
                            var sensor2Data = { categories: [], line1values: [], line2values: [] };
                            var sensor3Data = { categories: [], line1values: [], line2values: [] };
                            var sensor4Data = { categories: [], line1values: [], line2values: [] };
                            enginesTelemetry.engine1telemetry.forEach(function (reading) {
                                var timestamp = new Date(reading.timestamp);
                                sensor1Data.categories.push(timestamp);
                                sensor2Data.categories.push(timestamp);
                                sensor3Data.categories.push(timestamp);
                                sensor4Data.categories.push(timestamp);
                                sensor1Data.line1values.push(reading.sensor1);
                                sensor2Data.line1values.push(reading.sensor2);
                                sensor3Data.line1values.push(reading.sensor3);
                                sensor4Data.line1values.push(reading.sensor4);
                            });
                            enginesTelemetry.engine2telemetry.forEach(function (reading) {
                                var timestamp = new Date(reading.timestamp);
                                sensor1Data.categories.push(timestamp);
                                sensor2Data.categories.push(timestamp);
                                sensor3Data.categories.push(timestamp);
                                sensor4Data.categories.push(timestamp);
                                sensor1Data.line2values.push(reading.sensor1);
                                sensor2Data.line2values.push(reading.sensor2);
                                sensor3Data.line2values.push(reading.sensor3);
                                sensor4Data.line2values.push(reading.sensor4);
                            });
                            this.sensor1Data(sensor1Data);
                            this.sensor2Data(sensor2Data);
                            this.sensor3Data(sensor3Data);
                            this.sensor4Data(sensor4Data);
                            setTimeout(this.getTelemetryData, 1000);
                        };
                        Dashboard.prototype.onTelemetryLoadError = function () {
                            this.errorMessages.push(PredictiveMaintenance.Constants.couldNotLoadTelemetry);
                        };
                        Dashboard.prototype.getPredictionData = function () {
                            var getPredictionPromise = this.httpClient.get("api/prediction");
                            getPredictionPromise.done(this.onPredictionDataLoaded);
                            getPredictionPromise.fail(this.onPredictionLoadError);
                        };
                        Dashboard.prototype.onPredictionDataLoaded = function (prediction) {
                            var initialData = { categories: [], line1values: [], line2values: [] };
                            var rulData = initialData;
                            prediction.engine1prediction.forEach(function (reading) {
                                var cycles = reading.cycles;
                                rulData.categories.push(cycles);
                                rulData.line1values.push(reading.rul);
                            });
                            prediction.engine2prediction.forEach(function (reading) {
                                var cycles = reading.cycles;
                                rulData.categories.push(cycles);
                                rulData.line2values.push(reading.rul);
                            });
                            this.rulData(rulData);
                            if (prediction.engine1prediction.length > 0) {
                                var engine1PredictionRul = _.last(prediction.engine1prediction).rul;
                                this.engine1Rul(engine1PredictionRul.toString());
                                this.engine1Cycles(_.last(prediction.engine1prediction).cycles.toString());
                                this.engine1RulWarning(engine1PredictionRul < this.warningTreshold);
                            }
                            else {
                                this.engine1Rul(this.emptyString);
                                this.engine1Cycles(this.emptyString);
                                this.engine1RulWarning(false);
                            }
                            if (prediction.engine2prediction.length > 0) {
                                var engine2PredictionRul = _.last(prediction.engine2prediction).rul;
                                this.engine2Rul(_.last(prediction.engine2prediction).rul.toString());
                                this.engine2Cycles(_.last(prediction.engine2prediction).cycles.toString());
                                this.engine2RulWarning(engine2PredictionRul < this.warningTreshold);
                            }
                            else {
                                this.engine2Rul(this.emptyString);
                                this.engine2Cycles(this.emptyString);
                                this.engine2RulWarning(false);
                            }
                            setTimeout(this.getPredictionData, 1000);
                        };
                        Dashboard.prototype.onPredictionLoadError = function () {
                            this.errorMessages.push(PredictiveMaintenance.Constants.couldNotLoadPredictions);
                        };
                        Dashboard.prototype.onSendCommandError = function () {
                            this.errorMessages.push(PredictiveMaintenance.Constants.couldNotSendCommand);
                            this.sendingCommand(false);
                        };
                        Dashboard.prototype.closeMessage = function (message) {
                            this.errorMessages.remove(message);
                        };
                        Dashboard.prototype.startSimulation = function () {
                            var _this = this;
                            var startEmulationPromise = this.httpClient.post("api/simulation/start", {});
                            this.sendingCommand(true);
                            startEmulationPromise.done(function (state) {
                                _this.sendingCommand(false);
                                _this.simulationState(state);
                            });
                            startEmulationPromise.fail(this.onSendCommandError);
                        };
                        Dashboard.prototype.stopSimulation = function () {
                            var _this = this;
                            var stopEmulationPromise = this.httpClient.post("api/simulation/stop", {});
                            this.sendingCommand(true);
                            stopEmulationPromise.done(function (state) {
                                _this.sendingCommand(false);
                                _this.simulationState(state);
                            });
                            stopEmulationPromise.fail(this.onSendCommandError);
                        };
                        return Dashboard;
                    }());
                    PredictiveMaintenance.Dashboard = Dashboard;
                })(PredictiveMaintenance = Applications.PredictiveMaintenance || (Applications.PredictiveMaintenance = {}));
            })(Applications = Devices.Applications || (Devices.Applications = {}));
        })(Devices = Azure.Devices || (Azure.Devices = {}));
    })(Azure = Microsoft.Azure || (Microsoft.Azure = {}));
})(Microsoft || (Microsoft = {}));
var Microsoft;
(function (Microsoft) {
    var Azure;
    (function (Azure) {
        var Devices;
        (function (Devices) {
            var Applications;
            (function (Applications) {
                var PredictiveMaintenance;
                (function (PredictiveMaintenance) {
                    var SimulationStates = (function () {
                        function SimulationStates() {
                        }
                        SimulationStates.stopped = "Stopped";
                        SimulationStates.running = "Started";
                        SimulationStates.starting = "Starting";
                        SimulationStates.stopping = "Stopping";
                        return SimulationStates;
                    }());
                    PredictiveMaintenance.SimulationStates = SimulationStates;
                })(PredictiveMaintenance = Applications.PredictiveMaintenance || (Applications.PredictiveMaintenance = {}));
            })(Applications = Devices.Applications || (Devices.Applications = {}));
        })(Devices = Azure.Devices || (Azure.Devices = {}));
    })(Azure = Microsoft.Azure || (Microsoft.Azure = {}));
})(Microsoft || (Microsoft = {}));
String.prototype.format = function () {
    var values = [];
    for (var i = 0; i < arguments.length; i++) {
        values[i - 0] = arguments[i];
    }
    var formatted = this;
    for (var i = 0; i < values.length; i++) {
        var regexp = new RegExp("\\{" + i + "\\}", "gi");
        if (values[i])
            formatted = formatted.replace(regexp, values[i]);
        else
            formatted = formatted.replace(regexp, "");
    }
    return formatted;
};
$(function () {
    ko.applyBindings(new Microsoft.Azure.Devices.Applications.PredictiveMaintenance.Dashboard());
});
var Microsoft;
(function (Microsoft) {
    var Azure;
    (function (Azure) {
        var Devices;
        (function (Devices) {
            var Applications;
            (function (Applications) {
                var PredictiveMaintenance;
                (function (PredictiveMaintenance) {
                    var JQueryHttpClient = (function () {
                        function JQueryHttpClient() {
                            this.get = this.get.bind(this);
                            this.post = this.post.bind(this);
                            this.delete = this.delete.bind(this);
                            this.onBeforeSend = this.onBeforeSend.bind(this);
                            this.handleErrorResponse = this.handleErrorResponse.bind(this);
                            this.handle401Response = this.handle401Response.bind(this);
                            this.urlFormat = "/{0}";
                        }
                        JQueryHttpClient.prototype.onBeforeSend = function (xhrObj) {
                        };
                        JQueryHttpClient.prototype.handleErrorResponse = function (xmlHttpRequest) {
                            switch (xmlHttpRequest.status) {
                                case 401: this.handle401Response(xmlHttpRequest);
                            }
                        };
                        JQueryHttpClient.prototype.handle401Response = function (xmlHttpRequest) {
                            window.location.reload();
                        };
                        JQueryHttpClient.prototype.get = function (url) {
                            var promise = jQuery.ajax({
                                url: this.urlFormat.format(url),
                                method: "GET",
                                beforeSend: this.onBeforeSend,
                                error: this.handleErrorResponse,
                                cache: false
                            });
                            return promise;
                        };
                        JQueryHttpClient.prototype.post = function (url, data) {
                            var promise = jQuery.ajax({
                                url: this.urlFormat.format(url),
                                method: "POST",
                                data: data,
                                beforeSend: this.onBeforeSend,
                                error: this.handleErrorResponse,
                                cache: false
                            });
                            return promise;
                        };
                        JQueryHttpClient.prototype.patch = function (url, data) {
                            var promise = jQuery.ajax({
                                url: this.urlFormat.format(url),
                                method: "PATCH",
                                data: data,
                                beforeSend: this.onBeforeSend,
                                error: this.handleErrorResponse,
                                cache: false
                            });
                            return promise;
                        };
                        JQueryHttpClient.prototype.put = function (url, data) {
                            var promise = jQuery.ajax({
                                url: this.urlFormat.format(url),
                                method: "PUT",
                                data: data,
                                beforeSend: this.onBeforeSend,
                                error: this.handleErrorResponse,
                                cache: false
                            });
                            return promise;
                        };
                        JQueryHttpClient.prototype.delete = function (url) {
                            var promise = jQuery.ajax({
                                url: this.urlFormat.format(url),
                                method: "DELETE",
                                beforeSend: this.onBeforeSend,
                                error: this.handleErrorResponse,
                                cache: false
                            });
                            return promise;
                        };
                        JQueryHttpClient.prototype.head = function (url) {
                            var promise = jQuery.ajax({
                                url: this.urlFormat.format(url),
                                method: "HEAD",
                                beforeSend: this.onBeforeSend,
                                error: this.handleErrorResponse,
                                cache: false
                            });
                            return promise;
                        };
                        return JQueryHttpClient;
                    }());
                    PredictiveMaintenance.JQueryHttpClient = JQueryHttpClient;
                })(PredictiveMaintenance = Applications.PredictiveMaintenance || (Applications.PredictiveMaintenance = {}));
            })(Applications = Devices.Applications || (Devices.Applications = {}));
        })(Devices = Azure.Devices || (Azure.Devices = {}));
    })(Azure = Microsoft.Azure || (Microsoft.Azure = {}));
})(Microsoft || (Microsoft = {}));
var Microsoft;
(function (Microsoft) {
    var Azure;
    (function (Azure) {
        var Devices;
        (function (Devices) {
            var Applications;
            (function (Applications) {
                var PredictiveMaintenance;
                (function (PredictiveMaintenance) {
                    var TelemetryLineChartBinding = (function () {
                        function TelemetryLineChartBinding() {
                            this.init = this.init.bind(this);
                            this.createDataView = this.createDataView.bind(this);
                        }
                        TelemetryLineChartBinding.prototype.createDataView = function (categories, line1values, line2values) {
                            var fieldExpr = powerbi.data.SQExprBuilder.fieldExpr({ column: { schema: "s", entity: "table1", name: "date" } });
                            var categoryIdentities = categories.map(function (value) {
                                var expr = powerbi.data.SQExprBuilder.equal(fieldExpr, powerbi.data.SQExprBuilder.dateTime(value));
                                return powerbi.data.createDataViewScopeIdentity(expr);
                            });
                            var dataViewMetadata = {
                                columns: [
                                    {
                                        displayName: "Date",
                                        queryName: "Date",
                                        isMeasure: true,
                                        type: powerbi.ValueType.fromDescriptor({ dateTime: true })
                                    },
                                    {
                                        displayName: "Engine 1",
                                        isMeasure: true,
                                        format: "0000.00",
                                        queryName: "engine1",
                                        type: powerbi.ValueType.fromDescriptor({ numeric: true }),
                                        objects: { dataPoint: { fill: { solid: { color: "#C02FB0" } } } }
                                    },
                                    {
                                        displayName: "Engine 2",
                                        isMeasure: true,
                                        format: "0000.00",
                                        queryName: "engine2",
                                        type: powerbi.ValueType.fromDescriptor({ numeric: true }),
                                        objects: { dataPoint: { fill: { solid: { color: "#00AAAD" } } } }
                                    }
                                ]
                            };
                            var columns = [
                                {
                                    source: dataViewMetadata.columns[1],
                                    values: line1values
                                },
                                {
                                    source: dataViewMetadata.columns[2],
                                    values: line2values
                                }
                            ];
                            var dataValues = powerbi.data.DataViewTransform.createValueColumns(columns);
                            var tableDataValues = categories.map(function (countryName, idx) { return [countryName, columns[0].values[idx], columns[1].values[idx]]; });
                            return {
                                metadata: dataViewMetadata,
                                categorical: {
                                    categories: [{
                                            source: dataViewMetadata.columns[0],
                                            values: categories,
                                            identity: categoryIdentities,
                                        }],
                                    values: dataValues
                                },
                                table: {
                                    rows: tableDataValues,
                                    columns: dataViewMetadata.columns
                                }
                            };
                        };
                        TelemetryLineChartBinding.prototype.init = function (element, valueAccessor) {
                            var _this = this;
                            $(element).addClass("chart-indicator-progress");
                            var sourceObservable = valueAccessor();
                            var lineChart = new PredictiveMaintenance.LineChart(element);
                            var onChartDataReceived = function (chartData) {
                                if (!chartData) {
                                    return;
                                }
                                var dataView = _this.createDataView(chartData.categories, chartData.line1values, chartData.line2values);
                                lineChart.updateChartData(dataView);
                                $(element).removeClass("chart-indicator-progress");
                            };
                            sourceObservable.subscribe(onChartDataReceived);
                            $(window).resize(function () {
                                lineChart.resizeViewport();
                            });
                            onChartDataReceived(sourceObservable());
                        };
                        return TelemetryLineChartBinding;
                    }());
                    PredictiveMaintenance.TelemetryLineChartBinding = TelemetryLineChartBinding;
                    var sensorLineChartBinding = new TelemetryLineChartBinding();
                    ko.bindingHandlers["telemetryLineChart"] = {
                        init: sensorLineChartBinding.init
                    };
                })(PredictiveMaintenance = Applications.PredictiveMaintenance || (Applications.PredictiveMaintenance = {}));
            })(Applications = Devices.Applications || (Devices.Applications = {}));
        })(Devices = Azure.Devices || (Azure.Devices = {}));
    })(Azure = Microsoft.Azure || (Microsoft.Azure = {}));
})(Microsoft || (Microsoft = {}));
//# sourceMappingURL=app.js.map