﻿using System;
using System.Collections.Generic;
using System.Threading.Tasks;
using Microsoft.ServiceBus.Messaging;
using CustomIoTSolution.EventProcessor.Helpers;
using CustomIoTSolution.EventProcessor.Models;
using CustomIoTSolution.Common.Models;
using CustomIoTSolution.Common.Helpers;

namespace CustomIoTSolution.EventProcessor.Processors
{
    public class TemperatureEventDataProcessor : IEventProcessor
    {
        // Adjust this value to your sensor readings
        private const double temperatureThreshold = 32.5;

        public Task OpenAsync(PartitionContext context)
        {
            Console.WriteLine($"Open: {context.EventHubPath}, Partition Id: {context.Lease.PartitionId}");

            return Task.FromResult<object>(null);
        }

        public Task CloseAsync(PartitionContext context, CloseReason reason)
        {
            Console.WriteLine($"Close: {context.EventHubPath}, Reason: {reason}");

            return Task.FromResult<object>(null);
        }

        public Task ProcessEventsAsync(PartitionContext context, IEnumerable<EventData> messages)
        {
            foreach (var message in messages)
            {
                try
                {
                    var temperatureEventData = TemperatureEventData.FromEventData(message);

                    DisplayTemperature(temperatureEventData);

                    CheckTemperature(temperatureEventData);
                }
                catch (Exception ex)
                {
                    Console.WriteLine(ex.Message);
                }
            }

            return Task.FromResult<object>(null);
        }

        private void DisplayTemperature(TemperatureEventData temperatureEventData)
        {
            if (temperatureEventData != null)
            {
                Console.WriteLine(temperatureEventData.AverageValue);
            }
            else
            {
                Console.WriteLine("Unknown structure of the event data");
            }
        }

        // Definitions of DisplayTemperature and CheckTemperature
        private async void CheckTemperature(TemperatureEventData temperatureEventData)
        {
            if (temperatureEventData.AverageValue >= temperatureThreshold)
            {
                await AzureStorageHelper.WriteAlertToAzureTable(new AlertEntity()
                {
                    AlertTemperature = temperatureEventData.AverageValue,
                    PartitionKey = temperatureEventData.DeviceId,
                    RowKey = Guid.NewGuid().ToString(),
                    Timestamp = DateTime.Now,
                    ETag = "*"
                });

                NotificationHelper.SendToast(temperatureEventData.AverageValue);
            }
        }
    }
}
