﻿using CustomIoTSolution.Common.Config;
using CustomIoTSolution.Common.Helpers;
using CustomIoTSolution.NotificationClient.Helpers;
using CustomIoTSolution.NotificationClient.ViewModels;
using Microsoft.WindowsAzure.Messaging;
using System;
using System.Threading.Tasks;
using Windows.Networking.PushNotifications;
using Windows.UI.Popups;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;

namespace CustomIoTSolution.NotificationClient
{
    public sealed partial class MainPage : Page
    {
        private PushNotificationChannel pushNotificationChannel;
        private NotificationClientViewModel notificationClientViewModel = new NotificationClientViewModel();        

        public MainPage()
        {
            InitializeComponent();
        }

        private async void ButtonRegister_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                pushNotificationChannel = await PushNotificationChannelManager.CreatePushNotificationChannelForApplicationAsync();

                var notificationHub = new NotificationHub(Configuration.NotificationHubPath, 
                    Configuration.DefaultListenSharedAccessSignatureConnectionString);

                var registration = await notificationHub.RegisterNativeAsync(pushNotificationChannel.Uri);

                notificationClientViewModel.RegistrationId = registration.RegistrationId;
            }
            catch (Exception ex)
            {
                await DisplayMessage("Registration error: " + ex.Message);
            }
        }          
        
        private async Task DisplayMessage(string message)
        {
            var messageDialog = new MessageDialog(message, Name);
            await messageDialog.ShowAsync();
        }

        private async void ButtonGetSensorReadings_Click(object sender, RoutedEventArgs e)
        {
            var mostRecentEntity = await AzureStorageHelper.GetMostRecentEntity(
                Configuration.TelemetryTableName, notificationClientViewModel.Offset);

            notificationClientViewModel.SensorReadings =
                EntityConverter.DynamicTableEntityToSensorReadings(mostRecentEntity);
        }

        private async void ButtonGetLastAlarmInfo_Click(object sender, RoutedEventArgs e)
        {
            var mostRecentEntity = await AzureStorageHelper.GetMostRecentEntity(
                Configuration.AlertsTableName, notificationClientViewModel.Offset);

            notificationClientViewModel.TemperatureAlertInfo = 
                EntityConverter.DynamicTableEntityToTemperatureAlertInfo(mostRecentEntity);
        }
    }
}
